@@ -4,7 +4,7 @@ use feature qw(say);
 
 package App::GitHubPullRequest;
 {
-  $App::GitHubPullRequest::VERSION = '0.1.0';
+  $App::GitHubPullRequest::VERSION = '0.2.0';
 }
 
 # ABSTRACT: Command-line tool to query GitHub pull requests
@@ -36,10 +36,11 @@ $0 [<command> <args> ...]
 
 Where command is one of these:
 
-  help           Show this page
-  list [<state>] Show all pull requests (state: open/closed)
-  show <number>  Show details for the specific pull request
-  patch <number> Fetch a properly formatted patch for the specific pull request
+  help              Show this page
+  list [<state>]    Show all pull requests (state: open/closed)
+  show <number>     Show details for the specified pull request
+  patch <number>    Fetch a properly formatted patch for specified pull request
+  checkout <number> Create tracking branch for specified pull request
 
   login [<user>] [<password>] Login to GitHub and receive an access token
   comment <number> [<text>]   Create a comment on the specified pull request
@@ -112,6 +113,80 @@ sub patch {
 }
 
 
+sub checkout {
+    my ($self, $number) = @_;
+    die("Please specify a pull request number.\n") unless $number;
+    my $pr = $self->_fetch_one($number);
+    die("Unable to fetch pull request $number.\n")
+        unless defined $pr;
+
+    # Get required contributor branch info
+    my $head_repo   = $pr->{'head'}->{'repo'}->{'git_url'};
+    my $head_branch = $pr->{'head'}->{'ref'};
+    my $head_user   = $pr->{'head'}->{'user'}->{'login'};
+
+    # Check if the remote already exists in our repo
+    my $head_remote;
+    foreach my $line ( _qx("git", "remote -v") ) {
+        my ($remote, $url, $type) = split /\s+/, $line;
+        next unless $type eq '(fetch)'; # only consider fetch remotes
+        if ( $url eq $head_repo ) {
+            $head_remote = $remote;
+            last;
+        }
+    }
+
+    if ( $head_remote ) {
+        # Remote already exists, try to update its state
+        unless ( _qx(qw(git branch --list -r), qq{"$head_remote/$head_branch"}) ) {
+            # Create a new tracking branch, because one doesn't already exist
+            my ($content, $rc) = _run_ext(
+                qw(git remote set-branches),
+                '--add',        # don't remove any other existing tracking branches
+                $head_remote,   # our remote's name/alias
+                $head_branch,   # the ref we want to track
+            );
+            die("git failed with error $rc when trying to add tracking branch"
+              . " to existing remote.\n")
+                if $rc != 0;
+        }
+
+        # Fetch changes from just added remote
+        say "Fetching changes from '$head_remote/$head_branch'";
+        my ($content, $rc) = _run_ext(
+            qw(git fetch),
+            $head_remote,
+        );
+        die("git failed with error $rc when trying to update remote.\n")
+            if $rc != 0;
+    }
+    else {
+        # Create and fetch the branch info if it doesn't exist already
+        $head_remote = $head_user;
+        my ($content, $rc) = _run_ext(
+            qw(git remote add),
+            '-f',                    # only fetch specific refs
+            '-t', $head_branch,      # add only a ref for this ref, not all
+            $head_remote,            # what we'll name our remote
+            $head_repo,              # URL to the head repo
+        );
+        die("git failed with error $rc when trying to add remote.\n")
+            if $rc != 0;
+    }
+
+    # Actually checkout the ref we just updated as pr/<number>
+    my ($content, $rc) = _run_ext(
+        qw(git checkout),
+        '-b', "pr/$number",
+        '--track', "$head_remote/$head_branch",
+    );
+    die("git failed with error $rc when trying to check out branch.\n")
+        if $rc != 0;
+
+    return 0;
+}
+
+
 sub close {
     my ($self, $number) = @_;
     die("Please specify a pull request number.\n") unless $number;
@@ -263,7 +338,7 @@ sub _find_github_remote {
         my ($remote, $url, $type) = split /\s+/, $line;
         next unless $type eq '(fetch)'; # only consider fetch remotes
         next unless $url =~ m/github\.com/; # only consider remotes to github
-        if ( $url =~ m{github.com[:/](.+)\.git$} ) {
+        if ( $url =~ m{github.com[:/](.+?)(?:\.git)?$} ) {
             $repo = $1;
             last;
         }
@@ -334,6 +409,7 @@ sub _run_ext {
     _require_binary($prg);
     CORE::open my $fh, "-|", @_ or die("Can't run command '$cmd': $!");
     my $stdout = join("", <$fh>);
+    CORE::close $fh;
     my $rc = $? >> 8; # exit code, see perldoc perlvar for details
     return $stdout, $rc;
 }
@@ -481,7 +557,7 @@ App::GitHubPullRequest - Command-line tool to query GitHub pull requests
 
 =head1 VERSION
 
-version 0.1.0
+version 0.2.0
 
 =head1 SYNOPSIS
 
@@ -489,6 +565,7 @@ version 0.1.0
     $ prq list closed # not shown by default
     $ prq show 7      # also includes comments
     $ prq patch 7     # can be piped to colordiff if you like colors
+    $ prq checkout 7  # create upstream tracking branch pr/7
     $ prq help
 
     $ prq login       # Get access token for commands below
@@ -542,6 +619,12 @@ comments.
 
 Shows the patch associated with the specified pull request number.
 
+=head2 checkout <number>
+
+Checks out the specified pull request in a dedicated tracking branch.
+If the remote repo is not already specified in your git config, it will be
+added and the branch in question will be fetched.
+
 =head2 close <number>
 
 Closes the specified pull request number. Be aware, you can't close a pull
